"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTypegooseQueryServiceProviders = void 0;
const core_1 = require("@nestjs-query/core");
const nestjs_typegoose_1 = require("nestjs-typegoose");
const mongoose_1 = require("mongoose");
const is_class_1 = require("is-class");
const services_1 = require("./services");
const isTypegooseClass = (item) => is_class_1.isClass(item);
const isTypegooseClassWithOptions = (item) => isTypegooseClass(item.typegooseClass);
core_1.AssemblerSerializer((obj) => obj.toObject({ virtuals: true }))(mongoose_1.Document);
function ensureProperInput(item) {
    if (isTypegooseClass(item)) {
        return { typegooseClass: item };
    }
    if (isTypegooseClassWithOptions(item)) {
        return item;
    }
    return undefined;
}
function createTypegooseQueryServiceProvider(model) {
    var _a;
    const inputModel = ensureProperInput(model);
    if (!inputModel) {
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        throw new Error(`Model definitions ${model} is incorrect.`);
    }
    const modelName = (_a = inputModel.typegooseClass) === null || _a === void 0 ? void 0 : _a.name;
    return {
        provide: core_1.getQueryServiceToken({ name: modelName }),
        useFactory(ModelClass) {
            // initialize default serializer for documents, this is the type that mongoose returns from queries
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            core_1.AssemblerSerializer((obj) => obj.toObject({ virtuals: true }))(ModelClass);
            return new services_1.TypegooseQueryService(ModelClass);
        },
        inject: [nestjs_typegoose_1.getModelToken(modelName)],
    };
}
const createTypegooseQueryServiceProviders = (models) => models.map((model) => createTypegooseQueryServiceProvider(model));
exports.createTypegooseQueryServiceProviders = createTypegooseQueryServiceProviders;
//# sourceMappingURL=providers.js.map